import sys
import logging

logging.basicConfig(level = logging.INFO)
logger = logging.getLogger(__name__)

def parse_args():
    nargs = len(sys.argv)
    if nargs == 1:
        logger.info('No arguments supplied.')
        return None
    
    args = sys.argv[1:]
    
    firstarg = args[0].strip().lower()
    if firstarg in ['-h', '--help']:
        logger.info('Command help requested.')
        return 'usage'
    
    argdict = {}
    ints = ['N', 'E0', 'I0', 'R0', 'D0', 'tau'] 
    reals = ['lambda', 'beta', 'gamma', 'sigma', 'mu']
    
    for a in args:
        tokens = a.split('=')
        if len(tokens) == 1:
            t = tokens[0].strip()
            argdict[t] = None
        elif len(tokens) == 2:
            t0 = tokens[0].strip()
            t1 = tokens[1].strip()
            
            if t0 in ints:
                try:
                    v = int(t1)
                    argdict[t0] = v
                except ValueError:
                    msg = 'Argument ' + t0 + ' must be an integer.'
                    raise ValueError(msg)
                else:
                    if v < 0:
                        raise ValueError('Value for argument: ' + t0 + 
                                         ' should be non-negative')        
            elif t0 in reals:
                try:
                    v = float(t1)
                    argdict[t0] = v
                except ValueError:
                    msg = 'Argument ' + t0 + ' must be a float.'
                    raise ValueError(msg)
                else:
                    if v <= 0:
                        raise ValueError('Value for argument: ' + t0 + 
                                         ' should be positive')
            else:
                argdict[t0] = t1 # Just in case there are non-int non-float args
        else:
            raise ValueError('Invalid input: ', a)
            
    logger.info('Arguments: ' + str(argdict))
    return argdict


from covimath.models import sir
import pytest

def test_peak():
    N = 2000
    I0 = 1
    R0 = 0

    model = sir.SIR(N=N, beta=0.2, gamma=0.1, I0=I0, R0=R0, tau=150)
    
    model.solve()
    
    p = model.peak()[1]
    
    assert pytest.approx(307, 1) == p
    
def test_method_seq1():
    with pytest.raises(ValueError):
        N = 2000
        I0 = 1
        R0 = 0
        model = sir.SIR(N=N, beta=0.2, gamma=0.1, I0=I0, R0=R0, tau=150)
        model.peak()
    
def test_method_seq2():
    with pytest.raises(ValueError):
        N = 2000
        I0 = 1
        R0 = 0
        model = sir.SIR(N=N, beta=0.2, gamma=0.1, I0=I0, R0=R0, tau=150)
        model.plot()


from covimath.models import seird
import pytest

def test_peak():
    N = 1000
    E0 = 1
    I0 = 1
    R0 = 0
    D0 = 0

    model = seird.SEIRD(N=N, beta=1.38, sigma = 0.19, gamma=0.34, mu=0.03,
                       E0=E0, I0=I0, R0=R0, D0=D0, tau=150)
    
    model.solve()
    
    p = model.peak()[1]
    
    assert pytest.approx(125, 1) == p
    
def test_method_seq():
    with pytest.raises(ValueError):
        N = 1000
        E0 = 1
        I0 = 1
        R0 = 0
        D0 = 0
        
        model = seird.SEIRD(N=N, beta=1.38, sigma = 0.19, gamma=0.34,
                           mu = 0.03, E0=E0, I0=I0, R0=R0, D0=D0, tau=150)
        model.plot()


#!/usr/bin/env python3

from math import exp
import numpy as np
import matplotlib.pyplot as plt
import logging

from ..utils.modelargs import parse_args

logging.basicConfig(level = logging.INFO)
logger = logging.getLogger(__name__)

# Susceptible -> Infected -> Susceptible model
class SIS:
    
    def __init__(self, N, lam, mu, gamma, I0, tau):
        logger.info('Intializing SIS model ...')
        self.N = N # Total population (assumed constant)
        self.lam = lam # Daily contact rate (for adequate contact)
        self.mu = mu # Proportionality constant of daily deaths (in each class)
        self.gamma = gamma # Proportionality constant of daily recovery
        self.I0 = I0 # Initial infected population (number, not fraction)
        self.S0 = N - I0 # Initial susceptible population (number, not fraction)
        self.tau = tau # Time window in days over which to model
        self.sigma = lam * 1.0 / (gamma + mu) # contact number
    
    # Closed form function : Infected population (fraction)
    def i(self, t):
        lam = self.lam
        gamma = self.gamma
        mu = self.mu
        sigma = self.sigma
        I0 = self.I0
        
        if sigma == 1:
            return 1.0 / (lam*t + 1.0/I0)
        else:
            num = exp((gamma + mu)*(sigma -1 )*t)
            den = (exp((gamma + mu)*(sigma -1 )*t)) * (sigma * 1.0 / 
                                                (sigma - 1)) + 1.0 / I0
            
            return num * 1.0 / den
    
    # Infected population (absolute number)
    def iabs(self, t):
        return self.i(t) * self.N
    
    # Closed form function : Susceptibile poputation (fraction)
    def s(self, t):
        return 1.0 - self.i(t)
    
    # Susceptible population (absolute number)
    def sabs(self, t):
        return self.s(t) * self.N
    
    # Plot the S and I curves
    def plot(self, absolute=False):
        logger.info('Plotting SIS model ...')
        T = np.linspace(0, self.tau, self.tau)
        N = self.N
        
        if absolute:
            I = [self.iabs(tim) for tim in T]
            S = [self.sabs(tim) for tim in T]
        else:
            I = [self.i(tim) for tim in T]
            S = [self.s(tim) for tim in T]
        
        Iarr = np.array(I)
        Sarr = np.array(S)
        
        fig = plt.figure()
        ax = fig.add_subplot(111)
        
        if absolute:
            slab = 'Susceptible Population'
            ilab = 'Infected Population'
        else:
            slab = 'Susceptible Fraction'
            ilab = 'Infected Fraction'
            
        ax.plot(T, Sarr, label=slab)
        ax.plot(T, Iarr, label=ilab)
        
        ax.set_xlabel('Days')
        if absolute:
            ax.set_ylabel('Number of People')
            ax.set_ylim(0, N+100)
        else:
            ax.set_ylabel('Fraction of Population')
            ax.set_ylim(0, 1.1)
            
        legend = ax.legend()
        
        plt.show()

def usage():
    usagestr0 = './sis.py N=<N> I0=<I0> lambda=<lambda> mu=<mu> gamma=<gamma> tau=<tau> --abs, where: \n'
    usagestr1 = 'N = total population, assumed constant \n'
    usagestr2 = 'I0 = initial number of infected people \n'
    usagestr3 = 'lambda = daily contact rate \n'
    usagestr4 = 'mu = proportionality constant of daily deaths \n'
    usagestr5 = 'gamma = proportionality constant of daily recovery \n'
    usagestr6 = 'tau = duration in days over which the model should be constructed. \n'
    usagestr7 = '--abs (optional) means a plot with absolute numbers (instead of fraction of population) is needed. \n'
    usagestr8 = 'Note: order of arguments passsed is not important. Any other argument will be ignored. \n'
    usagestr9 = '\nExample: ./sis.py N=1000 lambda=0.05 mu=0.15 gamma=0.1 I0=1 tau=30 \n'
    
    print(usagestr0 + usagestr1 + usagestr2 + usagestr3 + usagestr4 + 
            usagestr5 + usagestr6 + usagestr7 + usagestr8 + usagestr9)
        
if __name__ == "__main__":
    
    res = parse_args()
    if res is None:
        logger.error('No argument passed.')
        usage()
        quit()
        
    if res == 'usage':
        usage()
        quit()
        
    if isinstance(res, dict):
        miss = []
        if 'N' in res:
            N = res['N']
        else:
            miss.append('N')
            
        if 'lambda' in res:
            lam = res['lambda']
        else:
            miss.append('lambda')
            
        if 'mu' in res:
            mu = res['mu']
        else:
            miss.append('mu')
        
        if 'gamma' in res:
            gamma = res['gamma']
        else:
            miss.append('gamma')
            
        if 'I0' in res:
            I0 = res['I0']
        else:
            miss.append('I0')
            
        if 'tau' in res:
            tau = res['tau']
        else:
            miss.append('tau')
        
        plotabs = False
        if '--abs' in res:
            plotabs = True
            
        if len(miss) > 0:
            logger.error('Some required arguments are missing: ' + ','.join(miss))
            usage()
            quit()

    model = SIS(N=N, lam=lam, mu=mu, gamma=gamma, I0=I0, tau=tau)
    
    if plotabs:
        model.plot(absolute=True)
    else:
        model.plot()


from math import floor, ceil
from covimath.models import sis

def test_inf():
    model = sis.SIS(N=1000, lam=0.05, mu = 0.15, 
                gamma=1./10, I0=1, tau=150)
    i = model.i(20) # should be around 1.8%
    assert ceil(i * 100) == 2.0

def test_sus():
    model = sis.SIS(N=1000, lam=0.05, mu = 0.15, 
                gamma=1./10, I0=1, tau=150)
    s = model.s(20) # should be around 98%
    assert floor(s * 100) == 98


from covimath.paramest import sirparams
import numpy as np
import pytest

def test_beta():
    ni = np.array([5, 7, 11, 20, 30, 45, 75, 115, 155, 
                   220, 315, 540, 720, 950])
    _beta = sirparams.findbeta(ni)
    
    pytest.approx(0.4, 0.01) == _beta




#!/usr/bin/env python3

import numpy as np
from scipy.optimize import curve_fit

# Find the parameter beta (= daily contact rate) from data from a region
# Pass a numpy array 'numinfec', of number of infected people over a small 
# period (typically one or two weeks) at the onset of the infection there.
def findbeta(numinfec):
    I0 = numinfec[0]
    
    def func(t, beta):
        return I0 * np.exp(beta * t)
    
    dur = numinfec.shape[0]
    
    tdata = np.array([t for t in range(dur)])
    popt, pcov = curve_fit(func, tdata, numinfec)
    
    est_beta = popt[0]
    
    return est_beta


import pathlib
from setuptools import find_packages, setup

cd = pathlib.Path(__file__).parent

# The text of the README file
readme = (cd / "README.md").read_text()

# This call to setup() does all the work
setup(
    name="covimath",
    packages=find_packages(exclude=("tests",)),
    version="0.1.2",
    description="Some mathematical models on epidemiology / Covid-19 infections",
    long_description=readme,
    long_description_content_type="text/markdown",
    url="https://github.com/techyugadi/covimath",
    download_url='https://github.com/techyugadi/covimath/archive/refs/tags/v0.1.2.tar.gz',
    author="TechYugadi",
    author_email="techyugadi@gmail.com",
    license="MIT",
    classifiers=[
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.6",
    ],
    install_requires=["numpy", "scipy", "matplotlib"],
    entry_points={
        "console_scripts": [
            "covimath=covimath.__main__:main",
        ]
    },
)



#!/usr/bin/env python3
import numpy as np
from scipy.integrate import solve_ivp
from scipy.signal import find_peaks
import matplotlib.pyplot as plt
import logging

from ..utils.modelargs import parse_args

logging.basicConfig(level = logging.INFO)
logger = logging.getLogger(__name__)

# Susceptible -> Exposed -> Infected -> Recovered -> Dead model
class SEIRD:
    
    def __init__(self, N, beta, sigma, gamma, mu, E0, I0, R0, D0, tau):
        logger.info('Initializing SEIR model ...')
        self.N = N # Total population (assumed constant)
        self.beta = beta # Daily contact rate (for adequate contact)
        self.sigma = sigma # Incubation rate
        self.gamma = gamma # Proportionality constant of daily recovery
        self.mu = mu # mortality rate
        self.E0 = E0 # Initial exposed population (number, not a fraction)
        self.I0 = I0 # Initial infected population (number, not fraction)
        self.R0 = R0 # Initial recovered population (number, not fraction)
        self.D0 = D0 # Initial dead  (number, not a fraction)
        # Initial susceptible population (number, not a fraction)
        self.S0 = N - E0 - I0 - R0 - D0
        self.tau = tau # Time window in days over which to model
        self.t = np.linspace(0, self.tau, self.tau)
        self.S = None
        self.E = None
        self.I = None
        self.R = None
        self.D = None
    
    # Solve the differential equation for this model
    def solve(self):
        
        def diffeqns(t, y, N, beta, sigma, gamma, mu):
            S, E, I, R, D = y
            dSdt = -beta * S * I / N
            dEdt = beta * S * I / N - sigma * E
            dIdt = sigma * E - gamma * I - mu * I
            dRdt = gamma * I
            dDdt = mu * I
            return [dSdt, dEdt, dIdt, dRdt, dDdt]
        
        y0 = [self.S0, self.E0, self.I0, self.R0, self.D0] # Initial values
        
        # Solve differential equations
        tspan = [0, self.tau]
        sol = solve_ivp(diffeqns, tspan, y0, 
                        t_eval = self.t, 
                        args=(self.N, self.beta, self.sigma, self.gamma, self.mu))
        
        self.S = sol.y[0]
        self.E = sol.y[1]
        self.I = sol.y[2]
        self.R = sol.y[3]
        self.D = sol.y[4]
    
    # Find the peak infection (day, number infected)
    def peak(self):
        if self.I is None:
            logger.error('solve() method has not been invoked yet')
            raise ValueError("peak() method invoked before invoking solve()")
            
        p = find_peaks(self.I, height = 0)
        day = p[0][0]
        infec = p[1]['peak_heights'][0]
        return day, infec     

    # Plot the S, E, R and D curves        
    def plot(self):
        if self.S is None or self.I is None or self.R is None or self.D is None:
            logger.error('solve() method has not been invoked yet')
            raise ValueError("plot() method invoked before invoking solve()")
            
        logger.info('Plotting SEIR model ...')
        N = self.N
        S = self.S
        E = self.E
        I = self.I
        R = self.R
        D = self.D
        t = self.t
        
        fig = plt.figure()
        ax = fig.add_subplot(111)
        
        ax.plot(t, S, label='Susceptible Population')
        ax.plot(t, E, label='Exposed Population')
        ax.plot(t, I, label='Infected Population')
        ax.plot(t, R, label='Recovered Population')
        ax.plot(t, D, label='Dead')
        ax.set_xlabel('Days')
        ax.set_ylabel('Number of People')
        
        ax.set_ylim(0, N+100)
        
        legend = ax.legend()
        
        plt.show()

def usage():
    usagestr0 = './seird.py N=<N> E0=<E0> I0=<I0> R0=<R0> D0=<D0> beta=<beta> sigma=<sigma> gamma=<gamma> mu=<mu> tau=<tau>, where: \n'
    usagestr1 = 'N = total population, assumed constant \n'
    usagestr2 = 'E0 = initial number of people exposed \n'
    usagestr3 = 'I0 = initial number of infected people \n'
    usagestr4 = 'R0 = initial number of people recovered \n'
    usagestr5 = 'D0 = initial number of people dead \n'
    usagestr6 = 'beta = daily contact rate \n'
    usagestr7 = 'sigma = incubation rate \n'
    usagestr8 = 'gamma = proportionality constant of daily recovery \n'
    usagestr9 = 'mu = mortality rate \n'
    usagestr10 = 'tau = duration in days over which the model should be constructed. \n'
    usagestr11 = 'Note: order of arguments passsed is not important. Any other argument will be ignored. \n'
    usagestr12 = '\nExample: ./seird.py N=1000 E0=1 I0=1 R0=0 D0=0 beta=1.38 sigma=0.19 gamma=0.34 mu=0.03 tau=150 \n'
    
    print(usagestr0 + usagestr1 + usagestr2 + usagestr3 + usagestr4 + 
          usagestr5 + usagestr6 + usagestr7 + usagestr8 + 
          usagestr9 + usagestr10 + usagestr11 + usagestr12)
    
if __name__ == "__main__":
    
    res = parse_args()
    if res is None:
        logger.error('No argument passed.')
        usage()
        quit()
        
    if res == 'usage':
        usage()
        quit()
        
    if isinstance(res, dict):
        miss = []
        if 'N' in res:
            N = res['N']
        else:
            miss.append('N')
            
        if 'beta' in res:
            beta = res['beta']
        else:
            miss.append('beta')
            
        if 'sigma' in res:
            sigma = res['sigma']
        else:
            miss.append('gamma')
            
        if 'gamma' in res:
            gamma = res['gamma']
        else:
            miss.append('gamma')
            
        if 'mu' in res:
            mu = res['mu']
        else:
            miss.append('mu')
        
        if 'E0' in res:
            E0 = res['E0']
        else:
            miss.append('E0')
            
        if 'I0' in res:
            I0 = res['I0']
        else:
            miss.append('I0')
            
        if 'R0' in res:
            R0 = res['R0']
        else:
            miss.append('R0')
            
        if 'D0' in res:
            D0 = res['D0']
        else:
            miss.append('D0')
            
        if 'tau' in res:
            tau = res['tau']
        else:
            miss.append('tau')
        
        if len(miss) > 0:
            logger.error('Some required arguments are missing: ' + ','.join(miss))
            usage()
            quit()

    model = SEIRD(N=N, E0=E0, I0=I0, R0=R0, D0=D0, beta=beta, sigma=sigma, 
                 gamma=gamma, mu=mu, tau=tau)
    
    model.solve()
    
    model.plot()


from covimath.models import seir
import pytest

def test_peak():
    N = 1000
    E0 = 1
    I0 = 1
    R0 = 0

    model = seir.SEIR(N=N, beta=1.38, sigma = 0.19, gamma=0.34, 
                       E0=E0, I0=I0, R0=R0, tau=150)
    
    model.solve()
    
    p = model.peak()[1]
    
    assert pytest.approx(141, 1) == p
    
def test_method_seq():
    with pytest.raises(ValueError):
        N = 1000
        E0 = 1
        I0 = 1
        R0 = 0
        model = seir.SEIR(N=N, beta=1.38, sigma = 0.19, gamma=0.34, 
                           E0=E0, I0=I0, R0=R0, tau=150)
        model.plot()